
/*################################################################################################
 *##################                                                            ##################
 *##################                     Genetic engine                         ##################
 *##################                                                            ##################
 *##################                      Header  file                          ##################
 *##################                                                            ##################
 *################################################################################################
 */

/*
 * RIGEL: Representation-Independant Genetic Engine Library
 *
 * Copyright (C) 2005-2005 Yann LANDRIN-SCHWEITZER a.k.a. Varkhan
 *
 *
 * This library is free software;  you can redistribute it and/or modify it
 * under  the terms of the  GNU Lesser General Public License  as published
 * by the  Free Software Foundation;  either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This  library  is  distributed  in  the hope that it will be useful, but
 * WITHOUT   ANY   WARRANTY;   without   even   the   implied  warranty  of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library;  if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 *
 * Author:   Yann LANDRIN-SCHWEITZER
 * Contact:  varkhan@free.fr
 * Homepage: http://www.varkhan.net/
 */


#ifndef _RIGEL_GENETICS_H_
#define _RIGEL_GENETICS_H_


#include <RIGEL/Interface.h>
#include <RIGEL/Parameters.h>


struct S_RIGEL_Location {
	unsigned long ident ;
	RIGEL_Genotype indiv ;
	unsigned long repr ;
	double eval ;
	double zone ;
	double rate ;
	double conf ;
} ;

struct S_RIGEL_Ancestor {
	unsigned long ident ;
	unsigned long parn1 ;
	unsigned long parn2 ;
} ;

struct S_RIGEL_Population {
	RIGEL_Interface iface ;
	RIGEL_Parameters param ;
	unsigned long date ;
	unsigned long size ;
	unsigned long nwid ;
	struct S_RIGEL_Location * locs ;
	unsigned long hist ;
	struct S_RIGEL_Ancestor * ancs ;
} ;

typedef struct S_RIGEL_Population * RIGEL_Population ;


RIGEL_Population RIGEL_Initialize(RIGEL_Interface iface, RIGEL_Parameters param) ;
RIGEL_Population RIGEL_Evaluate(RIGEL_Population pop) ;
RIGEL_Population RIGEL_Generation(RIGEL_Population pop) ;
void RIGEL_Terminate(RIGEL_Population pop) ;

RIGEL_Parameters RIGEL_Getparameters(RIGEL_Population pop) ;
RIGEL_Population RIGEL_SetParameters(RIGEL_Population pop, RIGEL_Parameters param) ;

unsigned long RIGEL_GetIdentByRank(RIGEL_Population pop, unsigned long rank) ;
unsigned long RIGEL_GetIdentByEval(RIGEL_Population pop, double emin, double emax) ;
unsigned long RIGEL_GetGenealogy(RIGEL_Population pop, unsigned long ident, unsigned long * line, unsigned long gmax) ;

RIGEL_Genotype RIGEL_GetItemByIdent(RIGEL_Population pop, unsigned long ident) ;
RIGEL_Genotype RIGEL_GetItemByRank(RIGEL_Population pop, unsigned long rank) ;



#endif //_RIGEL_GENETICS_H_

