
/*################################################################################################
 *##################                                                            ##################
 *##################              Abstract genotype interface                   ##################
 *##################                                                            ##################
 *##################                      Header  file                          ##################
 *##################                                                            ##################
 *################################################################################################
 */

/*
 * RIGEL: Representation-Independant Genetic Engine Library
 *
 * Copyright (C) 2005-2005 Yann LANDRIN-SCHWEITZER a.k.a. Varkhan
 *
 *
 * This library is free software;  you can redistribute it and/or modify it
 * under  the terms of the  GNU Lesser General Public License  as published
 * by the  Free Software Foundation;  either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This  library  is  distributed  in  the hope that it will be useful, but
 * WITHOUT   ANY   WARRANTY;   without   even   the   implied  warranty  of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library;  if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 *
 * Author:   Yann LANDRIN-SCHWEITZER
 * Contact:  varkhan@free.fr
 * Homepage: http://www.varkhan.net/
 */





#ifndef _RIGEL_INTERFACE_H_
#define _RIGEL_INTERFACE_H_


#include <RIGEL/Parameters.h>

#ifndef RIGEL_Genotype_Object
#define RIGEL_Genotype_Object void
#endif

typedef RIGEL_Genotype_Object * RIGEL_Genotype ;

#define RIGEL_PARADIGM_NODELETE (0x1<< 0)
#define RIGEL_PARADIGM_MUTALLOC (0x1<< 1)
#define RIGEL_PARADIGM_XOVALLOC (0x1<< 2)
#define RIGEL_PARADIGM_MIGALLOC (0x1<< 3)

#define RIGEL_PARADIGM_EVSTATIC (0x1<< 4)


struct S_RIGEL_Interface {
	unsigned long Paradigm ;
	RIGEL_Genotype (* Initialize) (RIGEL_Parameters param) ;
	RIGEL_Genotype (* Mutate) (RIGEL_Genotype indiv, RIGEL_Parameters param) ;
	RIGEL_Genotype (* CrossOver) (RIGEL_Genotype indi1, RIGEL_Genotype indi2, RIGEL_Parameters param) ;
	RIGEL_Genotype (* Migrate) (RIGEL_Genotype indiv, RIGEL_Parameters param) ;
	RIGEL_Genotype (* Duplicate) (RIGEL_Genotype indiv) ;
	void (* Destroy) (RIGEL_Genotype indiv) ;
	double (* Evaluate) (RIGEL_Genotype indiv) ;
	long (* Read) (RIGEL_Genotype * indiv, char * str, unsigned long len) ;
	long (* Write) (RIGEL_Genotype indiv, char * str, unsigned long len) ;
} ;

typedef struct S_RIGEL_Interface RIGEL_Interface ;


RIGEL_Interface RIGEL_InitInterface() ;

RIGEL_Interface RIGEL_LoadInterface(char * module) ;


#endif //_RIGEL_INTERFACE_H_

