
/*################################################################################################
 *##################                                                            ##################
 *##################                   Genetic parameters                       ##################
 *##################                                                            ##################
 *##################                      Header  file                          ##################
 *##################                                                            ##################
 *################################################################################################
 */

/*
 * RIGEL: Representation-Independant Genetic Engine Library
 *
 * Copyright (C) 2005-2005 Yann LANDRIN-SCHWEITZER a.k.a. Varkhan
 *
 *
 * This library is free software;  you can redistribute it and/or modify it
 * under  the terms of the  GNU Lesser General Public License  as published
 * by the  Free Software Foundation;  either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This  library  is  distributed  in  the hope that it will be useful, but
 * WITHOUT   ANY   WARRANTY;   without   even   the   implied  warranty  of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library;  if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 *
 * Author:   Yann LANDRIN-SCHWEITZER
 * Contact:  varkhan@free.fr
 * Homepage: http://www.varkhan.net/
 */





#ifndef _RIGEL_PARAMETERS_H_
#define _RIGEL_PARAMETERS_H_

#include <stdarg.h>


// Genetic parameters structure

struct S_RIGEL_Parameters {
	
	// Genetic operators rates

	double rate_init ;		// Initialisation rate (mostly useless)
	double rate_mutn ;		// Mutation rate
	double rate_xovr ;		// Crossover rate
	double rate_migr ;		// Migration rate

	// Population and genetics

	unsigned long popu_size ;	// Total size of the population
	unsigned long popu_flag ;	// Genetic flags
	unsigned long popu_hist ;	// History depth
	unsigned long popu_elit ;	// Elite number
	unsigned long popu_parn ;	// Number of parents individuals (not included elite)
	unsigned long popu_chld ;	// Number of children individuals (not included elite)
	unsigned long popu_init ;	// Number of recreated individuals

	double popu_fadj ;		// Adjustment to fitness levels (roulette/ranking)
	
	// Additionnal params ("mostly harmless")
	unsigned long parmem ;
	unsigned long parnum ;
	char * * parkey ;
	double * parval ;
} ;

typedef struct S_RIGEL_Parameters * RIGEL_Parameters ;


// Genetic parameters flags

// If set, minimizes the evaluation
#define RIGEL_PARAM_SORTDESC (0x1L<< 0)

// Selection methods: if not set, deterministic
#define RIGEL_PARAM_SELPROPN (0x1L<< 8)
#define RIGEL_PARAM_SELRANKG (0x1L<< 9)
#define RIGEL_PARAM_SELTOURN (0x1L<<10)
// Selection method mask
#define RIGEL_PARAM_SELECTOR (RIGEL_PARAM_SELPROPN|RIGEL_PARAM_SELRANKG|RIGEL_PARAM_SELTOURN)

// Replacement methods: if not set, deterministic
#define RIGEL_PARAM_REPPROPN (0x1L<<11)
#define RIGEL_PARAM_REPRANKG (0x1L<<12)
#define RIGEL_PARAM_REPTOURN (0x1L<<13)
// Replacement method mask
#define RIGEL_PARAM_REPLACMT (RIGEL_PARAM_REPPROPN|RIGEL_PARAM_REPRANKG|RIGEL_PARAM_REPTOURN)

// Selection strategy: if set, don't use elite in selection for parents
#define RIGEL_PARAM_SELCOMMA (0x1L<<16)
// Replacement strategy: if set, transmit elite without selection
#define RIGEL_PARAM_REPCOMMA (0x1L<<17)
// Strategy mask
#define RIGEL_PARAM_STRATEGY (RIGEL_PARAM_SELCOMMA|RIGEL_PARAM_REPCOMMA)



// Parameters store memory management
RIGEL_Parameters RIGEL_ParametersAlloc() ;
RIGEL_Parameters RIGEL_ParametersCopy(RIGEL_Parameters param) ;
void RIGEL_ParametersFree(RIGEL_Parameters param) ;

// Access to hash-stored parameters
double RIGEL_ParametersSetHash(RIGEL_Parameters param, char * name, double value) ;
double RIGEL_ParametersGetHash(RIGEL_Parameters param, char * name) ;
double RIGEL_ParametersDelHash(RIGEL_Parameters param, char * name) ;

// Initialisation and string-based setting of parameters
RIGEL_Parameters RIGEL_ParametersSetDefaults(RIGEL_Parameters param) ;
double RIGEL_ParametersGetDouble(RIGEL_Parameters param, char * name) ;
char * RIGEL_ParametersGetString(RIGEL_Parameters param, char * name) ;
unsigned long RIGEL_ParametersSet(RIGEL_Parameters param, char * name, char * val) ;

// Argument parsing
void RIGEL_ParametersParse(RIGEL_Parameters param, ...) ;
void RIGEL_ParametersParseArgs(RIGEL_Parameters param, int * argc, char * * * argv) ;

unsigned long RIGEL_ReadParameters(RIGEL_Parameters param, int fd) ;
unsigned long RIGEL_WriteParameters(RIGEL_Parameters param, int fd) ;




#endif //_RIGEL_PARAMETERS_H_

