
/*################################################################################################
 *##################                                                            ##################
 *##################                                                            ##################
 *##################                                                            ##################
 *##################                     Implementation                         ##################
 *##################                                                            ##################
 *################################################################################################
 */

/*
 * RIGEL: Representation-Independant Genetic Engine Library
 *
 * Copyright (C) 2005-2005 Yann LANDRIN-SCHWEITZER a.k.a. Varkhan
 *
 *
 * This library is free software;  you can redistribute it and/or modify it
 * under  the terms of the  GNU Lesser General Public License  as published
 * by the  Free Software Foundation;  either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This  library  is  distributed  in  the hope that it will be useful, but
 * WITHOUT   ANY   WARRANTY;   without   even   the   implied  warranty  of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library;  if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 *
 * Author:   Yann LANDRIN-SCHWEITZER
 * Contact:  varkhan@free.fr
 * Homepage: http://www.varkhan.net/
 */



#include <RIGEL/Genetics.h>
#include <RIGEL/FileFormat.h>

#include "Config.h"

//################################################################################
//###
//###  FileIO operations
//###


#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

unsigned long RIGEL_FReadParameters(RIGEL_Parameters param, FILE * is) {
	char eof = 0 ;
	char eol = 0 ;
	char * buf = NULL ;
	char * var ;
	char * val ;
	unsigned long mem = 0 ;
	unsigned long len = 0 ;
	unsigned long n = 0 ;
	unsigned long i, j ;
	if(param==NULL) {
		errno = EINVAL ;
		DbgLog("RIGEL_FReadParameters","RIGEL_arg",0,"NULL parameters argument") ;
		return -1 ;
	}
	if(is==NULL) {
		errno = EINVAL ;
		DbgLog("RIGEL_FReadParameters","RIGEL_arg",0,"Invalid input stream") ;
		return -1 ;
	}
	while(!eof) {
		i = 0 ;
		while(!eol && !eof) {
			if(len>=mem) {
				buf = realloc(buf,mem+1024) ;
				if(buf==NULL) {
					DbgLog("RIGEL_FReadParameters","RIGEL_mem",0,NULL) ;
					return -1 ;
				}
				mem += 1024 ;
			}
			j = fread(buf+len,1,mem-len,is) ;
			if(j==-1) {
				free(buf) ;
				DbgLog("RIGEL_FReadParameters","RIGEL_mem",0,NULL) ;
				return -1 ;
			}
			if(j<(mem-len)) eof = 1 ;
			len += j ;
			while(i<len) {
				if(buf[i]=='\n') {
					eol = 1 ;
					break ;
				}
				i ++ ;
			}
		}
		// We have either an eol, or eof, and i is at the end of line / file
		buf[i] = '\0' ; i ++ ;
		j = 0 ;
		while(buf[j]==' ' && buf[j]!='\0') j ++ ;
		var = buf+j ;
		while(buf[j]!=' ' && buf[j]!='=' && buf[j]!='\0') j ++ ;
		if(buf[j]=='\0') continue ;
		buf[j] = '\0' ; j ++ ;
		while((buf[j]==' '||buf[j]=='=') && buf[j]!='\0') j ++ ;
		val = buf+j ;
		while(buf[j]!='\0') j ++ ;
		buf[j] = '\0' ;
		RIGEL_ParametersSet(param,var,val) ;
		n ++ ;
		// Clean up the buffer,moving the unused part at the beginning
		j = 0 ;
		while(i<len) {
			buf[j] = buf[i] ;
			i ++ ;
			j ++ ;
		}
		len = j ;
	}
	free(buf) ;
	return n ;
}

unsigned long RIGEL_FWriteParameters(RIGEL_Parameters param, FILE * os) {
#define CHARNUM 512
	char s[CHARNUM+1] ;
	char * p ;
	unsigned long i, n = 0 ;
	unsigned long t, r ;
	char f ;

#define PRINT_VAL(name,format) \
	t = snprintf(s,CHARNUM, #name " = " format "\n",param->name) ; \
	if(t==-1) { \
		DbgLog("RIGEL_FWriteParameters","RIGEL_sys",0,NULL) ; \
		return -1 ; \
	} \
	r = fwrite(s,1,t,os) ; \
	if(r==-1) { \
		DbgLog("RIGEL_FWriteParameters","RIGEL_sys",0,NULL) ; \
		return -1 ; \
	} \
	n += r ; \
	if(t>=CHARNUM || r<t) { \
		return n ; \
	}

	if(param==NULL) {
		errno = EINVAL ;
		DbgLog("RIGEL_FWriteParameters","RIGEL_arg",0,"NULL parameters argument") ;
		return -1 ;
	}
	if(os==NULL) {
		errno = EINVAL ;
		DbgLog("RIGEL_FWriteParameters","RIGEL_arg",0,"Invalid output stream") ;
		return -1 ;
	}
	PRINT_VAL(popu_size,"%ld") ;
	PRINT_VAL(popu_hist,"%ld") ;
	PRINT_VAL(popu_elit,"%ld") ;
	PRINT_VAL(popu_parn,"%ld") ;
	PRINT_VAL(popu_chld,"%ld") ;
	PRINT_VAL(popu_init,"%ld") ;

	t = 0 ;
	p = "popu_flag =" ;
	i = 0 ;
	while(p[i]!='\0' && t<CHARNUM) { s[t] = p[i] ; t ++ ; i ++ ; }
	f = ' ' ;
	if(t<CHARNUM && param->popu_flag&RIGEL_PARAM_SORTDESC) {
		s[t] = f ; t ++ ;
		p = "SORTDESC" ;
		i = 0 ;
		while(p[i]!='\0' && t<CHARNUM) { s[t] = p[i] ; t ++ ; i ++ ; }
		f = '|' ;
	}
	if(t<CHARNUM && param->popu_flag&RIGEL_PARAM_SELPROPN) {
		s[t] = f ; t ++ ;
		p = "SELPROPN" ;
		i = 0 ;
		while(p[i]!='\0' && t<CHARNUM) { s[t] = p[i] ; t ++ ; i ++ ; }
		f = '|' ;
	}
	else if(t<CHARNUM && param->popu_flag&RIGEL_PARAM_SELRANKG) {
		s[t] = f ; t ++ ;
		p = "SELRANKG" ;
		i = 0 ;
		while(p[i]!='\0' && t<CHARNUM) { s[t] = p[i] ; t ++ ; i ++ ; }
		f = '|' ;
	}
	else if(t<CHARNUM && param->popu_flag&RIGEL_PARAM_SELTOURN) {
		s[t] = f ; t ++ ;
		p = "SELTOURN" ;
		i = 0 ;
		while(p[i]!='\0' && t<CHARNUM) { s[t] = p[i] ; t ++ ; i ++ ; }
		f = '|' ;
	}
	if(t<CHARNUM && param->popu_flag&RIGEL_PARAM_REPPROPN) {
		s[t] = f ; t ++ ;
		p = "REPPROPN" ;
		i = 0 ;
		while(p[i]!='\0' && t<CHARNUM) { s[t] = p[i] ; t ++ ; i ++ ; }
		f = '|' ;
	}
	else if(t<CHARNUM && param->popu_flag&RIGEL_PARAM_REPRANKG) {
		s[t] = f ; t ++ ;
		p = "REPRANKG" ;
		i = 0 ;
		while(p[i]!='\0' && t<CHARNUM) { s[t] = p[i] ; t ++ ; i ++ ; }
		f = '|' ;
	}
	else if(t<CHARNUM && param->popu_flag&RIGEL_PARAM_REPTOURN) {
		s[t] = f ; t ++ ;
		p = "REPTOURN" ;
		i = 0 ;
		while(p[i]!='\0' && t<CHARNUM) { s[t] = p[i] ; t ++ ; i ++ ; }
		f = '|' ;
	}
	if(t<CHARNUM) { if(param->popu_flag&RIGEL_PARAM_SELCOMMA) {
		s[t] = f ; t ++ ;
		p = "SELCOMMA" ;
		i = 0 ;
		while(p[i]!='\0' && t<CHARNUM) { s[t] = p[i] ; t ++ ; i ++ ; }
		f = '|' ;
	}
	else {
		s[t] = f ; t ++ ;
		p = "SELPLUS" ;
		i = 0 ;
		while(p[i]!='\0' && t<CHARNUM) { s[t] = p[i] ; t ++ ; i ++ ; }
		f = '|' ;
	} }
	if(t<CHARNUM) { if(param->popu_flag&RIGEL_PARAM_REPCOMMA) {
		s[t] = f ; t ++ ;
		p = "REPCOMMA" ;
		i = 0 ;
		while(p[i]!='\0' && t<CHARNUM) { s[t] = p[i] ; t ++ ; i ++ ; }
		f = '|' ;
	}
	else {
		s[t] = f ; t ++ ;
		p = "REPPLUS" ;
		i = 0 ;
		while(p[i]!='\0' && t<CHARNUM) { s[t] = p[i] ; t ++ ; i ++ ; }
		f = '|' ;
	} }
	r = fwrite(s,1,t,os) ;
	if(r==-1) {
		DbgLog("RIGEL_FWriteParameters","RIGEL_sys",0,NULL) ;
		return -1 ;
	}
	n += r ;
	if(t>=CHARNUM || r<t) {
		return n ;
	}

	// Printing rates
	PRINT_VAL(rate_init,"%f") ;
	PRINT_VAL(rate_mutn,"%f") ;
	PRINT_VAL(rate_xovr,"%f") ;
	PRINT_VAL(rate_migr,"%f") ;

	// Printing misc
	i = 0 ;
	while(i<param->parnum) {
		t = snprintf(s,CHARNUM,"%s = %f\n",param->parkey[i],param->parval[i]) ;
		if(t==-1) {
			DbgLog("RIGEL_FWriteParameters","RIGEL_sys",0,NULL) ;
			return -1 ;
		}
		r = fwrite(s,1,t,os) ;
		if(r==-1) {
			DbgLog("RIGEL_FWriteParameters","RIGEL_sys",0,NULL) ;
			return -1 ;
		}
		n += r ;
		if(t>=CHARNUM || r<t) {
			return n ;
		}
		i ++ ;
	}
	return n ;
#undef CHARNUM
#undef PRINT_VAL
}



RIGEL_Population _RIGEL_PopulationAlloc(unsigned long size, unsigned long hist) ;

RIGEL_Population RIGEL_FileLoad(RIGEL_Interface iface, RIGEL_Parameters param, char * file) {
	RIGEL_Population pop ;
	int fd ;
	char eof = 0 ;
	char eol = 0 ;
	char * buf = NULL ;
	unsigned long mem = 0 ;
	unsigned long len = 0 ;
	unsigned long n = 0 ;
	unsigned long i, j, r ;
	if(file==NULL || file[0]=='\0') {
		return NULL ;
	}
	fd = open(file,O_RDONLY) ;
	if(fd==-1) {
		return NULL ;
	}
	pop = _RIGEL_PopulationAlloc(param->popu_size,param->popu_hist) ;
	if(pop==NULL) {
		return NULL ;
	}
	pop->iface = iface ;
	pop->param = param ;
	pop->date = 1 ;
	pop->nwid = 1 ;
	while(!eof && n<pop->size) {
		char * str ;
		i = 0 ;
		while(!eol && !eof) {
			if(len>=mem) {
				buf = realloc(buf,mem+1024) ;
				if(buf==NULL) {
					return NULL ;
				}
				mem += 1024 ;
			}
			j = read(fd,buf+len,mem-len) ;
			if(j==-1) {
				free(buf) ;
				return NULL ;
			}
			if(j<(mem-len)) eof = 1 ;
			len += j ;
			while(i<len) {
				if(buf[i]=='\n') {
					eol = 1 ;
					break ;
				}
				i ++ ;
			}
		}
		// We have either an eol, or eof, and i is at the end of line / file
		buf[i] = '\0' ;
		j = 0 ;
		pop->locs[n].ident = strtoul(buf+j,&str,16) ;
		if(pop->locs[n].ident>=pop->nwid) pop->nwid = pop->locs[n].ident + 1 ;
		if(str==(buf+j)) {
			free(buf) ;
			return NULL ;
		}
		j = buf-str ;
		pop->locs[n].repr = 0 ;
		pop->locs[n].eval = strtod(buf+j,&str) ;
		if(str==(buf+j)) {
			free(buf) ;
			return NULL ;
		}
		j = buf-str ;
		pop->locs[n].zone = strtod(buf+j,&str) ;
		if(str==(buf+j)) {
			free(buf) ;
			return NULL ;
		}
		j = buf-str ;
		pop->locs[n].rate = strtod(buf+j,&str) ;
		if(str==(buf+j)) {
			free(buf) ;
			return NULL ;
		}
		j = buf-str ;
		pop->locs[n].conf = strtod(buf+j,&str) ;
		if(str==(buf+j)) {
			free(buf) ;
			return NULL ;
		}
		j = buf-str ;
		r = iface.Read(&(pop->locs[n].indiv),buf+j,i-j) ;
		n ++ ;
		// Clean up the buffer,moving the unused part at the beginning
		i ++ ;
		j = 0 ;
		while(i<len) {
			buf[j] = buf[i] ;
			i ++ ;
			j ++ ;
		}
		len = j ;
	}
	free(buf) ;
	// If shorter than expected, complete population with randomly generated individuals
	while(n<pop->size) {
		pop->locs[n].ident = pop->nwid ++ ;
		pop->locs[n].indiv = iface.Initialize(param) ;
		pop->locs[n].repr = 0 ;
		pop->locs[n].eval = 0.0 ;
		pop->locs[n].zone = 0.0 ;
		pop->locs[n].rate = 0.0 ;
		pop->locs[n].conf = 0.0 ;
		n ++ ;
	}
	return pop ;
}

unsigned long RIGEL_FileSave(RIGEL_Population pop, char * file) {
	int fd ;
	unsigned long n = 0 ;
	unsigned long i, j, r ;
	char * buf = NULL ;
	unsigned long mem = 0 ;
	if(pop==NULL) {
		DbgLog("RIGEL_FileSave","RIGEL_arg",0,"NULL population argument") ;
		return -1 ;
	}
	if(file==NULL) {
		DbgLog("RIGEL_FileSave","RIGEL_arg",0,"NULL file name") ;
		return -1 ;
	}
	if(pop->iface.Write==NULL) {
		return 0 ;
	}
	if(file[0]=='\0') fd = 1 ;
	else fd = open(file,O_WRONLY|O_CREAT|O_TRUNC) ;
	if(fd==-1) {
		return -1 ;
	}
	if(mem<64) {
		buf = realloc(buf,64) ;
		if(buf==NULL) {
			DbgLog("RIGEL_FileSave","RIGEL_mem",0,NULL) ;
			if(file[0]!='\0') close(fd) ;
			return -1 ;
		}
		mem = 64 ;
	}
	i = 0 ;
	while(i<pop->size) {
		// Write one more individual
		j = 0 ;
		if(j<mem) {
			r = snprintf(buf+j,mem-j,"%05lX ",pop->locs[i].ident) ;
			j += r ;
		}
		if(j<mem) {
			r = snprintf(buf+j,mem-j,"% -8.4f ",pop->locs[i].eval) ;
			j += r ;
		}
		if(j<mem) {
			r = snprintf(buf+j,mem-j,"% -8.4f ",pop->locs[i].zone) ;
			j += r ;
		}
		if(j<mem) {
			r = snprintf(buf+j,mem-j,"% -8.4f ",pop->locs[i].rate) ;
			j += r ;
		}
		if(j<mem) {
			r = snprintf(buf+j,mem-j,"% -8.4f ",pop->locs[i].conf) ;
			j += r ;
		}
//		DbgLog("RIGEL_FileSave","",4,"Write indiv: %u (%u->%u)",i,j,mem) ;
		r = write(fd,buf,j) ;
		if(r==-1) {
			DbgLog("RIGEL_FileSave","RIGEL_sys",0,NULL) ;
			free(buf) ;
			return -1 ;
		}
		n += r ;
		if(r<j) {
			DbgLog("RIGEL_FileSave","RIGEL_sys",0,NULL) ;
			free(buf) ;
			return n ;
		}
		j = pop->iface.Write(pop->locs[i].indiv,buf,mem) ;
		if(j==-1) {
			DbgLog("RIGEL_FileSave","RIGEL_sys",0,NULL) ;
			free(buf) ;
			return -1 ;
		}
		if(j>mem) {
			buf = realloc(buf,j);
			if(buf==NULL) {
				DbgLog("RIGEL_FileSave","RIGEL_mem",0,NULL) ;
				if(file[0]!='\0') close(fd) ;
				return -1 ;
			}
			mem = j ;
			j = pop->iface.Write(pop->locs[i].indiv,buf,mem) ;
			if(j==-1) {
				DbgLog("RIGEL_FileSave","RIGEL_sys",0,NULL) ;
				free(buf) ;
				return -1 ;
			}
		}
		buf[j] = '\n' ;
//		DbgLog("RIGEL_FileSave","",4,"Write test: %u",j) ;
		r = write(fd,buf,j) ;
		if(r==-1) {
			DbgLog("RIGEL_FileSave","RIGEL_sys",0,NULL) ;
			free(buf) ;
			return -1 ;
		}
		n += r ;
		if(r<j) {
			DbgLog("RIGEL_FileSave","RIGEL_sys",0,NULL) ;
			free(buf) ;
			return n ;
		}
		r = write(fd,"\n",1) ;
		if(r==-1) {
			DbgLog("RIGEL_FileSave","RIGEL_sys",0,NULL) ;
			free(buf) ;
			return -1 ;
		}
		n += 1 ;
		if(r<1) {
			DbgLog("RIGEL_FileSave","RIGEL_sys",0,NULL) ;
			free(buf) ;
			return n ;
		}

		i ++ ;
	}
	free(buf) ;
	if(file[0]!='\0') close(fd) ;
	return n ;
}





